################################################################################
#                                                                              #
#      Reproducing Figure 1                                                    #
#                                                                              #
#      Author: S. Janitza <janitza@ibe.med.uni-muenchen.de>                    #
#                                                                              #
################################################################################


# Set working directory where data is stored

setwd("Data/")


# Load training and validation data

train_data <- read.csv("training_data.csv", header = TRUE, sep = ",", as.is = TRUE)
valid_data <- read.csv("validation_data.csv", header = TRUE, sep = ",", as.is = TRUE)

# Keep only the ten discriminating variables 

ten_disc <- c("EIH2", "H2S", "M35", "X35Xe", "M36", "M64", "SO2Xe", "M66", "X76Xe", "X80Xe")

# variable names correspondence to the terminology in the paper:
#
# "EIH2"  -> H_2
# "H2S"   -> 34   
# "M35"   -> 35
# "X35Xe" -> 35* 
# "M36"   -> 36  
# "M64"   -> 64 
# "SO2Xe" -> 64*
# "M66"   -> 66 
# "X76Xe" -> 76*
# "X80Xe" -> 80*

train_data <- train_data[, c("Stamm", ten_disc)]
valid_data <- valid_data[, c("Stamm", ten_disc)]

# Significant differences between Gram-positive and Gram-negative

pvalues_train <- apply(train_data[, names(train_data)!= "Stamm"], 2, function(x){
  ks.test(x[train_data$Stamm == "pos"], x[train_data$Stamm == "neg"])$p.value})

pvalues_valid <- apply(valid_data[, names(valid_data)!= "Stamm"], 2, function(x){
  ks.test(x[valid_data$Stamm == "pos"], x[valid_data$Stamm == "neg"])$p.value})

# adjust baseline for plotting (negative values in EIH2 (H_2) only due to technical problems)

train_data$EIH2 <- train_data$EIH2 - min(train_data$EIH2, valid_data$EIH2) + 10
valid_data$EIH2 <- valid_data$EIH2 - min(train_data$EIH2, valid_data$EIH2) + 10


# Create Figure 1 (training set)

setEPS()
#postscript("Molecules_identified_training.eps", horizontal = FALSE, onefile = FALSE, paper = "special", height = 6.5, width = 8)
postscript("Molecules_identified.eps", horizontal = FALSE, onefile = FALSE, paper = "special", height = 6.5, width = 15)
par(mfrow = c(1, 2))

boxplot(train_data$EIH2[train_data$Stamm == "neg"],  train_data$EIH2[train_data$Stamm == "pos"],
        train_data$H2S[train_data$Stamm == "neg"],   train_data$H2S[train_data$Stamm == "pos"],
        train_data$M35[train_data$Stamm == "neg"],   train_data$M35[train_data$Stamm == "pos"],
        train_data$X35Xe[train_data$Stamm == "neg"], train_data$X35Xe[train_data$Stamm == "pos"],
        train_data$M36[train_data$Stamm == "neg"],   train_data$M36[train_data$Stamm == "pos"],
        train_data$M64[train_data$Stamm == "neg"],   train_data$M64[train_data$Stamm == "pos"],
        train_data$SO2Xe[train_data$Stamm == "neg"], train_data$SO2Xe[train_data$Stamm == "pos"],
        train_data$M66[train_data$Stamm == "neg"],   train_data$M66[train_data$Stamm == "pos"],
        train_data$X76Xe[train_data$Stamm == "neg"], train_data$X76Xe[train_data$Stamm == "pos"],
        train_data$X80Xe[train_data$Stamm == "neg"], train_data$X80Xe[train_data$Stamm == "pos"], 
        col = c("white", "gray70"), yaxt = "n", xaxt = "n", frame.plot = FALSE, 
        log = "y", ylim = c(10, 10^7), main = "Training set",
        xlab = "Identified discriminators", ylab = "Signal intensity [cps]")

labels = c(expression("H"[2]), "34", "35", "35*", "36", "64", "64*", "66", "76*", "80*")
axis(1, at =  2*(1:10) - 0.5, labels = labels)

axis.at <- 10 ^ c(1:7) #definiert die grossen Ticks der y-Achse
axis(2, las = 1, at = axis.at,  labels = parse(text = paste("10^", 1:7, sep = ""))) #zeichnet die grossen Ticks und die y-Achse
axis(2, at = 1:10 * rep(axis.at[-length(axis.at)], each = 10), tcl = -0.2, labels = FALSE, cex.axis=0.6)

#legend("topright", legend = c("Gram-negative", "Gram-positive"), fill = c("white", "grey70"))


pnames <- pvalues_train
pnames <- round(pvalues_train, digits = 3) * (round(pvalues_train, digits = 3) != 0)

pnames[pnames == 0] <- as.numeric(substr(format(pvalues_train[pnames == 0], scientific = T), 
                                         start = nchar(format(pvalues_train[pnames == 0], scientific = T)) - 1, 
                                         stop = nchar(format(pvalues_train[pnames == 0], scientific = T)))) - 1
sapply(1:10, function(z){
  if(pnames[ten_disc[z]] >=1){
    text(z*2-0.5, 8, bquote(p<10^.(-pnames[ten_disc[z]])), cex = 0.8)
  }else{
    text(z*2-0.5, 8, paste("p=", pnames[ten_disc[z]], sep = ""), cex = 0.8)
  }
})

#graphics.off()


# Create Figure 1 (validation set)

#setEPS()
#postscript("Molecules_identified_validation.eps", horizontal = FALSE, 
#           onefile = FALSE, paper = "special", height = 6.5, width = 8)

boxplot(valid_data$EIH2[valid_data$Stamm == "neg"],  valid_data$EIH2[valid_data$Stamm == "pos"],
        valid_data$H2S[valid_data$Stamm == "neg"],   valid_data$H2S[valid_data$Stamm == "pos"],
        valid_data$M35[valid_data$Stamm == "neg"],   valid_data$M35[valid_data$Stamm == "pos"],
        valid_data$X35Xe[valid_data$Stamm == "neg"], valid_data$X35Xe[valid_data$Stamm == "pos"],
        valid_data$M36[valid_data$Stamm == "neg"],   valid_data$M36[valid_data$Stamm == "pos"],
        valid_data$M64[valid_data$Stamm == "neg"],   valid_data$M64[valid_data$Stamm == "pos"],
        valid_data$SO2Xe[valid_data$Stamm == "neg"], valid_data$SO2Xe[valid_data$Stamm == "pos"],
        valid_data$M66[valid_data$Stamm == "neg"],   valid_data$M66[valid_data$Stamm == "pos"],
        valid_data$X76Xe[valid_data$Stamm == "neg"], valid_data$X76Xe[valid_data$Stamm == "pos"],
        valid_data$X80Xe[valid_data$Stamm == "neg"], valid_data$X80Xe[valid_data$Stamm == "pos"], 
        col = c("white", "gray70"), yaxt = "n", xaxt = "n", frame.plot = FALSE, 
        log = "y", ylim = c(10, 10^7), main = "Validation set",
        xlab = "Identified discriminators", ylab = "Signal intensity [cps]")

labels = c(expression("H"[2]), "34", "35", "35*", "36", "64", "64*", "66", "76*", "80*")
axis(1, at =  2*(1:10) - 0.5, labels = labels)

axis.at <- 10 ^ c(1:7) #definiert die grossen Ticks der y-Achse
axis(2, las = 1, at = axis.at,  labels = parse(text = paste("10^", 1:7, sep = ""))) #zeichnet die grossen Ticks und die y-Achse
axis(2, at = 1:10 * rep(axis.at[-length(axis.at)], each = 10), tcl = -0.2, labels = FALSE, cex.axis=0.6)

legend("topright", legend = c("Gram-negative", "Gram-positive"), fill = c("white", "grey70"))


pnames <- pvalues_valid
pnames <- round(pvalues_valid, digits = 3) * (round(pvalues_valid, digits = 3) != 0)

pnames[pnames == 0] <- as.numeric(substr(format(pvalues_valid[pnames == 0], scientific = T), 
                                         start = nchar(format(pvalues_valid[pnames == 0], scientific = T)) - 1, 
                                         stop = nchar(format(pvalues_valid[pnames == 0], scientific = T)))) - 1
sapply(1:10, function(z){
  if(pnames[ten_disc[z]] >=1){
    text(z*2-0.5, 8, bquote(p<10^.(-pnames[ten_disc[z]])), cex = 0.8)
  }else{
    text(z*2-0.5, 8, paste("p=", pnames[ten_disc[z]], sep = ""), cex = 0.8)
  }
})

graphics.off()



