################################################################################
#                                                                              #
#      Comparison of classification methods for anaerobe sample                #
#                                                                              #
#      Author: S. Janitza <janitza@ibe.med.uni-muenchen.de>                    #
#                                                                              #
################################################################################

# Load packages

require("CMA")


# Load R objects ('classification' - objects produced by CMA and rankings by Gini VIM)

load("R_Objects/all_classifiers.Rda")
load("R_Objects/sel_RF.Rda")


# Load training data

training_data <- read.csv("Data/training_data.csv", header = TRUE, sep = ",", as.is = TRUE)
pred          <- training_data[, !names(training_data) %in% c("Probe", "Stamm")] # only predictors


# Description of 'classification' - objects:

# classifiers built from 5-fold CV (100 iterations)
# "presel" - objects correspond to classifiers based on a subset of preselected predictors
# "presel" - objects of length 10 or below: preselection of 10, 20, 30, 40, ..., 100 predictors (or below 100 if classifier cannot handle so many predictors) 
# "presel" - objects of length 20: preselction of 5, 10, 15, 20, 25, ..., 100 predictors
# preselection by t test, limma or random forest Gini importance


###############################################################
#                                                             #
#          Compare 'classification' - objects:                #
#                                                             #
###############################################################

# Note: error rate of 0.2906977% if classifying all observations as Gram-positive

# plot cross-validated error rate for classifiers (also shown in the supplementary material) 
# Figure is stored as pdf-file in the current working directory 

graphics.off()
pdf("comparison_1.pdf", width = 6, height = 15)
par(mfrow = c(1, 1), mar = c(5, 12.5, 0, 2))

(compar_mis_all    <- compare(rev(unlist(
  list(list(res_boost_bin),
       list(res_boost_exp),
       list(res_boost_quad),
       list(res_gbm_bern),
       list(res_gbm_ada),
       list(res_rf),
       list(res_plsRf),
       res_rf_presel,
       list(res_svmLinear),
       list(res_svmPoly),
       list(res_svmRadial),
       list(res_lasso),
       list(res_plr),
       list(res_enet),
       list(res_knn),
       res_knn_presel_ttest, 
       res_knn_presel_limma,
       list(res_pknn),
       res_pknn_presel_ttest,
       res_pknn_presel_limma,
       res_nnet_presel_ttest,
       res_nnet_presel_limma), recursive = FALSE)),     
  measure = "misclassification", plot = TRUE,
  main = "", xlab = "Prediction error", 
  yaxt = "n", frame.plot = FALSE, horizontal = TRUE, las = 1))

labels <- rev(c("Gradient boosting (bin.)","Gradient boosting (exp.)", "Gradient boosting (quad.)", 
            "Tree-based boosting (bern.)", "Tree-based boosting (exp.)", 
            "RF", "RF (PLS)", 
            "RF (p = 10)", "RF (p = 20)", "RF (p = 30)", "RF (p = 40)", "RF (p = 50)",
            "RF (p = 60)", "RF (p = 70)", "RF (p = 80)", "RF (p = 90)", "RF (p = 100)",
            "SVM (linear)", "SVM (polynomial)", "SVM (radial)",
            "Lasso", "Ridge", "Elastic net", "k n.n.",
            "k n.n. (p = 10, t-test)", "k n.n. (p = 20, t-test)", "k n.n. (p = 30, t-test)", "k n.n. (p = 40, t-test)", "k n.n. (p = 50, t-test)",
            "k n.n. (p = 60, t-test)", "k n.n. (p = 70, t-test)", "k n.n. (p = 80, t-test)", "k n.n. (p = 90, t-test)", "k n.n. (p = 100, t-test)",
            "k n.n. (p = 10, Limma)", "k n.n. (p = 20, Limma)", "k n.n. (p = 30, Limma)", "k n.n. (p = 40, Limma)", "k n.n. (p = 50, Limma)",
            "k n.n. (p = 60, Limma)", "k n.n. (p = 70, Limma)", "k n.n. (p = 80, Limma)", "k n.n. (p = 90, Limma)", "k n.n. (p = 100, Limma)",
            "p. k n.n.",
            "p. k n.n. (p = 10, t-test)", "p. k n.n. (p = 20, t-test)", "p. k n.n. (p = 30, t-test)", "p. k n.n. (p = 40, t-test)", "p. k n.n. (p = 50, t-test)",
            "p. k n.n. (p = 60, t-test)", "p. k n.n. (p = 70, t-test)", "p. k n.n. (p = 80, t-test)", "p. k n.n. (p = 90, t-test)", "p. k n.n. (p = 100, t-test)",
            "p. k n.n. (p = 10, Limma)", "p. k n.n. (p = 20, Limma)", "p. k n.n. (p = 30, Limma)", "p. k n.n. (p = 40, Limma)", "p. k n.n. (p = 50, Limma)",
            "p. k n.n. (p = 60, Limma)", "p. k n.n. (p = 70, Limma)", "p. k n.n. (p = 80, Limma)", "p. k n.n. (p = 90, Limma)", "p. k n.n. (p = 100, Limma)",
            "Neural netw. (p = 10, t-test)", "Neural netw. (p = 20, t-test)", "Neural netw. (p = 30, t-test)", "Neural netw. (p = 40, t-test)", "Neural netw. (p = 50, t-test)",
            "Neural netw. (p = 60, t-test)", "Neural netw. (p = 70, t-test)", "Neural netw. (p = 80, t-test)", "Neural netw. (p = 90, t-test)", "Neural netw. (p = 100, t-test)",
            "Neural netw. (p = 10, Limma)", "Neural netw. (p = 20, Limma)", "Neural netw. (p = 30, Limma)", "Neural netw. (p = 40, Limma)", "Neural netw. (p = 50, Limma)",
            "Neural netw. (p = 60, Limma)", "Neural netw. (p = 70, Limma)", "Neural netw. (p = 80, Limma)", "Neural netw. (p = 90, Limma)", "Neural netw. (p = 100, Limma)"))

axis(2, at =  1:85, labels = labels, las = 2)

graphics.off()


# (cont.) plot cross-validated error rate for classifiers (also shown in the supplementary material) 
# Figure is stored as pdf-file in the current working directory 


pdf("comparison_2.pdf", width = 6, height = 15)
par(mfrow = c(1, 1), mar = c(5, 12.5, 0, 2))


(compar_mis_all    <- compare(rev(unlist(list(
  res_lda_presel_ttest,
  res_lda_presel_limma,
  list(res_plsLda),
  res_plsLda_presel_ttest,
  res_plsLda_presel_limma,
  res_fda_presel_ttest,
  res_fda_presel_limma), recursive = FALSE)),     
  measure = "misclassification", plot = TRUE,
  main = "", xlab = "Prediction error", 
  yaxt = "n", frame.plot = FALSE, horizontal = TRUE, las = 1))

labels <- rev(c(paste("LDA (p = ", 2*(1:20), "), t-test", sep = ""),
                paste("LDA (p = ", 2*(1:20), "), Limma", sep = ""),
                "LDA (PLS)",
                paste("LDA (p = ", 10*(1:10), "), t-test, PLS", sep = ""),
                paste("LDA (p = ", 10*(1:10), "), Limma, PLS", sep = ""),
                paste("Fisher's LDA (p = ", 10*(1:6), "), t-test", sep = ""),
                paste("Fisher's LDA (p = ", 10*(1:6), "), Limma", sep = "")))

axis(2, at =  1:length(labels), labels = labels, las = 2)

graphics.off()


# (cont.) plot cross-validated error rate for classifiers (also shown in the supplementary material) 
# Figure is stored as pdf-file in the current working directory 

pdf("comparison_3.pdf", width = 6, height = 12)
par(mfrow = c(1, 1), mar = c(5, 12.5, 0, 2))

(compar_mis_all    <- compare(rev(unlist(list(
  list(res_dlda),
  res_dlda_presel_ttest,
  res_dlda_presel_limma,
  res_qda_presel_ttest,
  res_qda_presel_limma), recursive = FALSE)),     
  measure = "misclassification", plot = TRUE,
  main = "", xlab = "Prediction error", 
  yaxt = "n", frame.plot = FALSE, horizontal = TRUE, las = 1))

labels <- rev(c("DLDA",
                paste("DLDA (p = ", 5*(1:20), "), t-test", sep = ""),
                paste("DLDA (p = ", 5*(1:20), "), Limma", sep = ""),
                paste("QDA (p = ", 2*(1:9), "), t-test", sep = ""),
                paste("QDA (p = ", 2*(1:9), "), Limma", sep = "")
                ))

axis(2, at =  1:length(labels), labels = labels, las = 2)

graphics.off()
                




# Object res_rf_presel is a list of length 10 with RF 'classification' - objects with previous variable selection 
# using the best 10 (1st list element), 20 (2nd list element), 30 (3rd list element), 40, etc. predictors


# -> Random Forest with 10 preselected predictors by Gini variable importance measure has lowest error rate with 0.08953

# Important values of the best 'classification' - object (on training data):

evaluation(res_rf_presel[[1]], measure = "misclassification")   # Error rate: 0.09
evaluation(res_rf_presel[[1]], measure = "sensitivity")         # Amount of rightly identified Gram-positives (sensitivity): 0.979
evaluation(res_rf_presel[[1]], measure = "specifity")           # Amount of rightly identified Gram-negatives (specificity): 0.742
evaluation(res_rf_presel[[1]], measure = "auc")                 # Area under Curve: 0.928




###############################################################
#                                                             #
#          Best discriminating variables. Top lists           #
#                                                             #
###############################################################

# Top list of variables with highest Gini importance measure. Selection frequencies of 100 * 5 CV folds

sel_top5 <- sort(table(unlist(sel_RF@rankings[[1]])[,1:5]), decreasing = TRUE)/500          # Selection frequencies of top 5 predictors 
dimnames(sel_top5) <- list(dimnames(pred)[[2]][c(as.numeric(unlist(dimnames(sel_top5))))])
sel_top6 <- sort(table(unlist(sel_RF@rankings[[1]])[,1:6]), decreasing = TRUE)/500          # Selection frequencies of top 6 predictors
dimnames(sel_top6) <- list(dimnames(pred)[[2]][c(as.numeric(unlist(dimnames(sel_top6))))])
sel_top7 <- sort(table(unlist(sel_RF@rankings[[1]])[,1:7]), decreasing = TRUE)/500          # Selection frequencies of top 7 predictors
dimnames(sel_top7) <- list(dimnames(pred)[[2]][c(as.numeric(unlist(dimnames(sel_top7))))])
sel_top8 <- sort(table(unlist(sel_RF@rankings[[1]])[,1:8]), decreasing = TRUE)/500          # Selection frequencies of top 8 predictors
dimnames(sel_top8) <- list(dimnames(pred)[[2]][c(as.numeric(unlist(dimnames(sel_top8))))])
sel_top9 <- sort(table(unlist(sel_RF@rankings[[1]])[,1:9]), decreasing = TRUE)/500          # Selection frequencies of top 9 predictors
dimnames(sel_top9) <- list(dimnames(pred)[[2]][c(as.numeric(unlist(dimnames(sel_top9))))])
sel_top10 <- sort(table(unlist(sel_RF@rankings[[1]])[,1:10]), decreasing = TRUE)/500        # Selection frequencies of top 10 predictors
dimnames(sel_top10) <- list(dimnames(pred)[[2]][c(as.numeric(unlist(dimnames(sel_top10))))])

# -> stable top lists indicating strong discriminative ability of variables
# Note: the best 10 predictors for the final prediction rule are derived from a separate analysis (see file 'deriving_final_RF_prediction_rule')


