################################################################################
#                                                                              #
#      Deriving the random forest prediction rule based on the training data   #
#                                                                              #
#      Author: S. Janitza <janitza@ibe.med.uni-muenchen.de>                    #
#                                                                              #
################################################################################


# For reproducing results please install the following R version and packages:

# - R version 2.13.1 (64 bit)
# - CMA version 1.10.0
# - Biobase version 2.12.2
# - randomForest version 4.6-2


# Load packages

require("CMA")
require("randomForest")


# Load training data

training_data <- read.csv("Data/training_data.csv", header = TRUE, sep = ",", as.is = TRUE)
pred          <- as.matrix(training_data[, !names(training_data) %in% c("Probe", "Stamm")]) # only predictors
resp          <- as.factor(training_data$Stamm)


###############################################################
#                                                             #
#  Deriving the RF prediction rule on the whole training data #
#                                                             #
###############################################################

# Deriving RF prediction rule based on the best 10 predictors on the training data

# Selection of the best 10 predictors (using Gini variable importance measure)

sel_RF_pred_model           <- GeneSelection(X = pred, y = resp, method = "rf", seed = 123)
sel_top10_pred_model_number <- sel_RF_pred_model@rankings[[1]][1:10]
sel_top10_pred_model        <- dimnames(pred)[[2]][c(as.numeric(sel_top10_pred_model_number))]


sel_top10_pred_model        # show the best 10 predictors


# Parameter (mtry and nodesize) tuning (3 fold CV)

set.seed(123)
tune_RF_pred_model <- CMA::tune(X          = pred, 
                                y          = resp, 
                                CV         = 3, 
                                genesel    = sel_RF_pred_model, 
                                nbgene     = 10, 
                                grids      = list(mtry     = ceiling(c(0.1, 0.25, 0.5, 1, 2) * sqrt(10)), 
                                                  nodesize = c(1, 2, 3)), 
                                classifier = rfCMA)

# Extract optimal parameters for mtry and nodesize

mtry     <- tune_RF_pred_model@hypergrid[which.min(tune_RF_pred_model@tuneres[[1]]),]$mtry
nodesize <- tune_RF_pred_model@hypergrid[which.min(tune_RF_pred_model@tuneres[[1]]),]$nodesize


# Fit final prediction model using 10 best predictors and optimal tuning parameters

res_RF_pred_model <- randomForest(x        = pred[,sel_top10_pred_model_number], 
                                  y        = resp, 
                                  mtry     = mtry, 
                                  ntree    = 1000, 
                                  nodesize = nodesize, 
                                  seed     = 123)


# Save prediction rule as R-object

save(res_RF_pred_model, file = "R_Objects/RF_prediction_rule.Rda")
